from typing import Any
from pydantic import Field
from .common import Common


class Object(Common):
    attributed_to: str = Field(
        examples=["https://actor.example/"],
        description="id of the actor that authored this object",
        alias="attributedTo",
    )
    content: str = Field(description="The content of the object")
    summary: str | None = Field(
        None,
        description="The summary of the object",
    )
    name: str | None = Field(
        None,
        description="The name of the object",
    )
    attachment: list[dict[str, Any]] | None = Field(
        None,
        description="A list of objects that are attached to the original object",
    )
    tag: list[dict[str, Any]] | None = Field(
        None,
        description="A list of objects that expand on the content of the object",
    )
    url: list[dict[str, Any] | str] | None = Field(
        None,
        description="A list of urls that expand on the content of the object",
    )
    sensitive: bool | None = Field(
        None,
        description="""
    Marks the object as sensitive. Currently, used by everyone, a better way would be an element of the tag list that labels the object as sensitive due a reason
    """,
    )
    in_reply_to: str | None = Field(
        None,
        description="The object being replied to. Currently a string. Not sure if this is what I want.",
        alias="inReplyTo",
    )
    context: str | None = Field(None, description="The context of the object")



from bovine.activitystreams.utils import id_for_object

from .list_utils import transform_to_list


def transform_url_part(url_part):
    if isinstance(url_part, str):
        url_part = {"type": "Link", "href": url_part}
    return url_part


def transform_url(url) -> list[dict] | None:
    """Transform to a list of links

    ```python
    >>> transform_url("http://remote.test/html")
    [{'type': 'Link', 'href': 'http://remote.test/html'}]

    ```
    """
    url_list = transform_to_list(url)

    return [transform_url_part(url_part) for url_part in url_list]


def transform_to_list_of_uris(data) -> list[str]:
    """Transforms to a list of URIs. As this simplifies a
    complicated data structure, some information may be lost.

    ``` python
    >>> transform_to_list_of_uris("http://remote.test/id")
    ['http://remote.test/id']

    >>> transform_to_list_of_uris({"id": "http://remote.test/id"})
    ['http://remote.test/id']

    >>> transform_to_list_of_uris({"ignored": True})
    []

    ```
    """
    to_list = transform_to_list(data)

    return [x for x in (id_for_object(to) for to in to_list) if x]



import re
import pytest

from muck_out.types import Actor
from muck_out.types.validated.actor import PropertyValue

from .actor import normalize_property_value, normalize_actor, actor_stub


def test_normalize_actor_failure():
    assert normalize_actor({}) is None


actor_base = {
    "@context": "https://www.w3.org/ns/activitystreams",
    "type": "Person",
    "id": "https://example.com/users/alice",
}


@pytest.mark.parametrize(
    "data",
    [
        {
            "@context": "https://www.w3.org/ns/activitystreams",
            "type": "Person",
            "id": "https://example.com/users/alice",
            "inbox": "https://example.com/users/alice/inbox",
        },
        {
            "@context": "https://www.w3.org/ns/activitystreams",
            "type": "Person",
            "id": "https://example.com/users/alice",
            "inbox": "https://example.com/users/alice/inbox",
            "icon": {
                "type": "Image",
                "url": "https://example.com/users/alice/icon.png",
            },
        },
    ],
)
def test_normalize_actor(data):
    assert isinstance(normalize_actor(data), Actor)


def to_snake_case(string: str):
    return re.sub("([A-Z]+)", r"_\1", string).lower()


@pytest.mark.parametrize(
    "field, value, mapped",
    [
        ("summary", "<p>Hello, world!</p>", "<p>Hello, world!</p>"),
        (
            "summary",
            "<p>Test<script>alert('hi')</script></p>",
            "<p>Test</p>",
        ),
        ("name", "Alice", "Alice"),
        (
            "alsoKnownAs",
            ["https://example.com/users/alice"],
            ["https://example.com/users/alice"],
        ),
    ],
)
def test_normalize_actor_field(field: str, value: str, mapped: str):
    data = actor_base.copy()
    data[field] = value
    actor = normalize_actor(data)
    assert getattr(actor, to_snake_case(field)) == mapped


actor_object = {
    "@context": [
        "https://www.w3.org/ns/activitystreams",
    ],
    "id": "http://mitra/users/admin",
    "type": "Person",
    "name": None,
    "preferredUsername": "admin",
    "inbox": "http://mitra/users/admin/inbox",
    "outbox": "http://mitra/users/admin/outbox",
    "followers": "http://mitra/users/admin/followers",
    "following": "http://mitra/users/admin/following",
    "subscribers": "http://mitra/users/admin/subscribers",
    "featured": "http://mitra/users/admin/collections/featured",
    "assertionMethod": [
        {
            "id": "http://mitra/users/admin#main-key",
            "type": "Multikey",
            "controller": "http://mitra/users/admin",
            "publicKeyMultibase": "z4MXj1wBzi9jUstyPBJ7qsa7G5waXG8McxXkjvAVHFLMxhGiJTGXK1EWseFfrUaH6Btt7TCSB1Entgg5SyAaHc1Ssh698puSozC41J2no8rgpMVRPzFVBoAYuNamM5FW9qCP1XV2y1cJ2y3gmuoreUVyn1jgW7Gb3NxsEcxjfs3fE1SzXjCnkxqVikYjkByqbXeB4AvYeXP1wjS1KB9xzKAzAjnRrui6RLHi6sCji2f7kdmxALcmjMbLyKjmYozhoG5ZqVkfNNZuAf4PLgyJQ84rDNKUGRgZeCeNh31hLLMUsSFYgpArfp813dbusq3A5BnnWN24rQtrj7nN9gQsxYjSTRhLpQcSLnwCQxJ1Sc2u9zfDwcJdW",
        },
        {
            "id": "http://mitra/users/admin#ed25519-key",
            "type": "Multikey",
            "controller": "http://mitra/users/admin",
            "publicKeyMultibase": "z6Mkgo38JzBJ6pLDyNz6YhnDN3ZXU6GcBWpp2c5dCEXcJtwW",
        },
    ],
    "publicKey": {
        "id": "http://mitra/users/admin#main-key",
        "owner": "http://mitra/users/admin",
        "publicKeyPem": "-----BEGIN PUBLIC KEY-----\nMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAqh38rZnS2RDft6RAGt5O\np4teGYSYmaeB2l5UGRCpnzcVf3m4v/x6LpXJXG8bDkNNnIZ/OzZ8tNPzKZJ6sKpI\nngayjowI/YzN8BBPJuKtaB3B4ImNrvG865OQl0h+TtfXaNTRWK8U108bMW2HH2/n\nTWJiaYiCcxLPzFRPLLd/TG6IYgeSKlRpYJ5TOdpZg1Hcx7/eRljFvdq7wmUcbM2W\nXmUYv1sjmwGQ92VCEy8d0biE/IesHdE6QM/5ALdAcQGtTn5QK7AEe43VmklkTfV+\nqDs38r3kVAMuHxnpASj+nuMp7Y7dRm92aKJ1vj93HxXTE1Ajh2eQYk8ulHQjcLky\nGwIDAQAB\n-----END PUBLIC KEY-----\n",
    },
    "generator": {
        "type": "Application",
        "implements": [
            {
                "name": "RFC-9421: HTTP Message Signatures",
                "href": "https://datatracker.ietf.org/doc/html/rfc9421",
            },
            {
                "name": "RFC-9421 signatures using the Ed25519 algorithm",
                "href": "https://datatracker.ietf.org/doc/html/rfc9421#name-eddsa-using-curve-edwards25",
            },
        ],
    },
    "manuallyApprovesFollowers": False,
    "url": "http://mitra/users/admin",
    "published": "2025-09-11T13:34:15.718558Z",
    "updated": "2025-09-11T13:34:15.718558Z",
}


def test_actor_import():
    result = actor_stub(actor_object)

    assert result.url

    assert result.preferred_username == "admin"


def test_normalize_property_value_dict():
    data = {"type": "Other", "name": "wooo"}

    assert normalize_property_value(data) == data


def test_normalize_property_value():
    data = {
        "type": "PropertyValue",
        "name": "Blog",
        "value": '<a href="http://value.example/" target="_blank" rel="nofollow noopener noreferrer me" translate="no"><span class="invisible">http://</span><span class="">value.example/</span><span class="invisible"></span></a>',
    }

    result = normalize_property_value(data)

    assert isinstance(result, PropertyValue)

    assert result.value == "http://value.example/"



from pydantic import Field

from muck_out.validators import HtmlStringOrNone, IdFieldOrNone, UrlList

from .common import CommonAll


class ActorStub(CommonAll):
    """Describes an ActivityPub actor"""

    inbox: IdFieldOrNone = Field(
        None,
        examples=["https://actor.example/inbox"],
        description="The inbox of the actor",
    )

    outbox: IdFieldOrNone = Field(
        None,
        examples=["https://actor.example/outbox"],
        description="The outbox of the actor",
    )

    icon: dict | None = Field(
        None,
        examples=[{"type": "Image", "url": "https://actor.example/icon.png"}],
        description="The avatar of the actor",
    )

    summary: HtmlStringOrNone = Field(
        None,
        examples=["My Fediverse account"],
        description="Description of the actor",
    )

    name: HtmlStringOrNone = Field(
        None,
        examples=["Alice"],
        description="Display name of the actor",
    )

    also_known_as: list[str] | None = Field(
        None,
        examples=[["https://alice.example", "https://alice.example/profile"]],
        alias="alsoKnownAs",
        description="Other uris associated with the actor",
    )

    url: UrlList = Field(
        default=[],
        description="A list of urls that expand on the content of the object",
    )

    preferred_username: str | None = Field(
        None, examples=["john"], alias="preferredUsername"
    )

    # attachments: list[dict | PropertyValue] | None = Field(
    #     None, description="""attachments ... currently used for property values"""
    # )



"""Routines to normalize an ActivityPub activity or object.

The routines here take a dictionary and turn them into another
one."""

import logging

from bovine.activitystreams.utils import id_for_object

from muck_out.types import Activity, ActivityStub
from muck_out.transform.list_utils import list_from_value

from .object import normalize_object

from .base import normalize_to, normalize_id

logger = logging.getLogger(__name__)


def activity_stub(data: dict) -> ActivityStub:
    """Builds the activity stub"""
    return ActivityStub.model_validate(data)


def normalize_activity(activity: dict, actor: str | None = None) -> Activity:
    """
    Normalizes activities.

    :param activity: The activity being normalized
    :param actor: Actor receiving this activity
    :returns:
    """
    try:
        obj = activity.get("object")
        if isinstance(obj, dict):
            try:
                obj = normalize_object(obj)
            except Exception:
                if isinstance(obj, dict):
                    obj = obj.get("id")

        return Activity.model_validate(
            {
                "@context": activity.get("@context"),
                "id": normalize_id(activity),
                "type": activity.get("type"),
                "actor": id_for_object(activity.get("actor")),
                "object": obj,
                "to": normalize_to(activity.get("to"), actor),
                "cc": list_from_value(activity.get("cc")),
                "published": activity.get("published"),
                "target": activity.get("target"),
                "content": activity.get("content"),
            }
        )
    except Exception as e:
        # logger.exception(e)
        # logger.info(activity)

        raise e



from typing import Any
from muck_out.transform.utils import sanitize_html
from muck_out.types import ObjectStub, Object


def object_stub(data: dict[str, Any]) -> ObjectStub:
    """Constructs a stub from data

    This function is not a direct equivalent to ObjectStub.model_validate
    as functionality happens that is not field to field for

    - `content` filled from `contentMap`

    """
    stub = ObjectStub.model_validate(data)

    if stub.content is None:
        content_map = data.get("contentMap")
        if isinstance(content_map, dict):
            values = content_map.values()
            if len(values) > 0:
                stub.content = sanitize_html(list(content_map.values())[0])

    return stub


def normalize_object(obj: dict[str, Any]) -> Object:
    """Normalizes an object

    :params obj: The object to be normalized
    :returns:
    """

    stub = object_stub(obj)
    result = Object.model_validate(stub.model_dump())
    result.field_context = stub.field_context

    return result



from typing import Any

from muck_out.transform.utils import remove_html
from muck_out.types import ActorStub, Actor
from muck_out.types.validated.actor import PropertyValue


def actor_stub(data: dict[str, Any]) -> ActorStub:
    """Returns the stub actor"""
    stub = ActorStub.model_validate(data)

    return stub


def normalize_property_value(data: dict[str, Any]) -> PropertyValue | dict[str, Any]:
    if data.get("type") == "PropertyValue":
        return PropertyValue.model_validate(
            {
                "name": remove_html(data.get("name")),
                "value": remove_html(data.get("value")),
            }
        )
    return data


def normalize_actor(data: dict[str, Any]) -> Actor | None:
    """Normalizes an ActivityPub actor"""

    try:
        stub = actor_stub(data)

        return Actor.model_validate(stub.model_dump(by_alias=True))

    except Exception:
        return None
